@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -21,10 +21,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -70,6 +66,7 @@ extern int errno;
 
 #include <lib_functions.h>
 #include <conf.h>
+#include <addrmatch.h>
 
 
 /*
@@ -410,12 +407,13 @@ const char *
 check_int (const char *s)
 {
 	char *ptr;
+	long int unused;
 
 #if HAVE_ASSERT_H
 	assert (s != NULL);
 #endif
 
-	strtol (s, &ptr, 0);
+	unused = strtol (s, &ptr, 0);
 	while (*ptr != 0 && isspace (*ptr) != 0) {
 		ptr++;
 	}
@@ -452,6 +450,47 @@ check_dirname (const char *s)
 	return NULL;
 }
 
+/*
+ * Check if the provided string contain an valid address family (any, inet
+ * or inet6)
+ *
+ * Return:
+ *   NULL if yes or an error message if no
+ */
+const char *
+check_addrfamily (const char *s)
+{
+#if HAVE_ASSERT_H
+	assert (s != NULL);
+#endif
+
+	if (	   schedwi_strcasecmp (s, "any") == 0
+		|| schedwi_strcasecmp (s, "inet") == 0
+		|| schedwi_strcasecmp (s, "inet6") == 0)
+	{
+		return NULL;
+	}
+	return _("the parameter value should be `any', `inet' (use IPv4 only), or `inet6' (use IPv6 only)");
+}
+
+/*
+ * Check if the provided network list is properly built.
+ *
+ * Return:
+ *   NULL if yes or an error message if no
+ */
+const char *
+check_netlist (const char *s)
+{
+#if HAVE_ASSERT_H
+	assert (s != NULL);
+#endif
+
+	if (addr_match_list (NULL, s) != 0) {
+		return _("Inconsistent mask length in one of the address");
+	}
+	return NULL;
+}
 
 /*
  * strtok (see strtok(3)).
@@ -567,7 +606,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 
 	clearerr (fd);
 	nb_read = fread (file_content, 1, file_info.st_size, fd);
-	if (nb_read != file_info.st_size && ferror (fd) != 0) {
+	if (nb_read != (size_t) file_info.st_size && ferror (fd) != 0) {
 		save_errno = errno;
 		free (file_content);
 		fclose (fd);
@@ -602,7 +641,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 		if (name_len == 0) {
 			/* ie. '= value' (missing parameter name) */
 			nb_errors++;
-			fprintf (stderr, _("%s: syntax error on line %d: "), 
+			fprintf (stderr, _("%s: syntax error on line %d: "),
 					file_name, num_line);
 			fputs (_("missing parameter name:\n"), stderr);
 			fputs (save_line, stderr);
@@ -617,7 +656,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 		if (*line != '=') {
 			/* ie. 'name  value' (missing '=') */
 			nb_errors++;
-			fprintf (stderr, _("%s: syntax error on line %d: "), 
+			fprintf (stderr, _("%s: syntax error on line %d: "),
 					file_name, num_line);
 			fputs (_("missing `=' sign between name and value:\n"),
 					stderr);
@@ -635,7 +674,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 		if (*line == '\0' || *line == '#' || *line == ';') {
 			/* ie. 'name =' (missing value) */
 			nb_errors++;
-			fprintf (stderr, _("%s: syntax error on line %d: "), 
+			fprintf (stderr, _("%s: syntax error on line %d: "),
 					file_name, num_line);
 			fputs (_("missing value:\n"), stderr);
 			fputs (save_line, stderr);
@@ -657,7 +696,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 		if (params[i].name == NULL) {
 			nb_errors++;
 			fprintf (stderr,
-				_("%s: unknown parameter name on line %d:\n"), 
+				_("%s: unknown parameter name on line %d:\n"),
 				file_name, num_line);
 			fputs (save_line, stderr);
 			fputc ('\n', stderr);
@@ -670,7 +709,7 @@ conf_init (const char *file_name, conf_param_t *params_array)
 			if (err_msg != 0) {
 				nb_errors++;
 				fprintf (stderr,
-			_("%s: erroneous value parameter on line %d: "), 
+			_("%s: erroneous value parameter on line %d: "),
 					file_name, num_line);
 				fputs (err_msg, stderr);
 				fputs (":\n", stderr);
