@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -19,10 +19,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -68,6 +64,7 @@ extern int errno;
 #include <lib_functions.h>
 #include <job_launcher.h>
 
+
 /*
  * System calls which trigger an error in the child process
  */
@@ -258,6 +255,7 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 	int status_pipe_child[2], out, err;
 	child_errcode_t child_err;
 	ssize_t nb_read;
+	ssize_t unused;
 #if HAVE_SETRLIMIT
 	struct rlimit rlim;
 #endif
@@ -281,9 +279,9 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 	 * process can report the error to its father.
 	 */
 	if (pipe (status_pipe_child) != 0) {
-		write (status_pipe_father[1], &errno, sizeof (int));
-		child_err = CHILD_PIPE; 
-		write (status_pipe_father[1], &child_err,
+		unused = write (status_pipe_father[1], &errno, sizeof (int));
+		child_err = CHILD_PIPE;
+		unused = write (status_pipe_father[1], &child_err,
 						sizeof (child_errcode_t));
 		close (status_pipe_father[1]);
 		exit (1);
@@ -291,9 +289,9 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 
 	child = fork ();
 	if (child < 0) {
-		write (status_pipe_father[1], &errno, sizeof (int));
-		child_err = CHILD_FORK; 
-		write (status_pipe_father[1], &child_err,
+		unused = write (status_pipe_father[1], &errno, sizeof (int));
+		child_err = CHILD_FORK;
+		unused = write (status_pipe_father[1], &child_err,
 						sizeof (child_errcode_t));
 		close (status_pipe_father[1]);
 		close (status_pipe_child[0]);
@@ -327,8 +325,8 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 
 		/* Send back the error code to the father */
 		waitpid (child, NULL, 0);
-		write (status_pipe_father[1], &result_errno, sizeof (int));
-		write (status_pipe_father[1], &child_err,
+		unused = write (status_pipe_father[1], &result_errno, sizeof (int));
+		unused = write (status_pipe_father[1], &child_err,
 						sizeof (child_errcode_t));
 		close (status_pipe_father[1]);
 		exit (1);
@@ -357,9 +355,9 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 	if (	   getpriority (PRIO_PROCESS, 0) != j->nice
 		&& setpriority (PRIO_PROCESS, 0, j->nice) != 0)
 	{
-		write (3, &errno, sizeof (int));
-		child_err = CHILD_SETPRIORITY; 
-		write (3, &child_err, sizeof (child_errcode_t));
+		unused = write (3, &errno, sizeof (int));
+		child_err = CHILD_SETPRIORITY;
+		unused = write (3, &child_err, sizeof (child_errcode_t));
 		close (3);
 		exit (1);
 	}
@@ -367,25 +365,22 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 
 #if HAVE_SETRLIMIT && HAVE_GETRLIMIT
 	/* ulimit */
-	if (getrlimit (RLIMIT_FSIZE, &rlim) != 0) {
-		write (3, &errno, sizeof (int));
-		child_err = CHILD_SETRLIMIT; 
-		write (3, &child_err, sizeof (child_errcode_t));
-		close (3);
-		exit (1);
-	}
-	if (j->lim == 0) {
-		j->lim = RLIM_INFINITY;
-	}
-	if (rlim.rlim_cur != j->lim) {
-		rlim.rlim_cur = j->lim;
-		rlim.rlim_max = j->lim;
-		if (setrlimit (RLIMIT_FSIZE, &rlim) != 0) {
-			write (3, &errno, sizeof (int));
-			child_err = CHILD_SETRLIMIT; 
-			write (3, &child_err, sizeof (child_errcode_t));
-			close (3);
-			exit (1);
+	if (getrlimit (RLIMIT_FSIZE, &rlim) == 0) {
+		if (j->lim == 0) {
+			j->lim = RLIM_INFINITY;
+		}
+		if (rlim.rlim_cur != j->lim) {
+			rlim.rlim_cur = j->lim;
+			rlim.rlim_max = j->lim;
+			if (setrlimit (RLIMIT_FSIZE, &rlim) != 0) {
+				unused = write (3, &errno, sizeof (int));
+				child_err = CHILD_SETRLIMIT;
+				unused = write (3,
+						&child_err,
+						sizeof (child_errcode_t));
+				close (3);
+				exit (1);
+			}
 		}
 	}
 #endif
@@ -394,24 +389,24 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 #if HAVE_SETREGID
 	if (setregid (j->gid, j->gid) != 0) {
 		child_err = CHILD_SETGID;
-		write (3, &errno, sizeof (int));
-		write (3, &child_err, sizeof (child_errcode_t));
+		unused = write (3, &errno, sizeof (int));
+		unused = write (3, &child_err, sizeof (child_errcode_t));
 		close (3);
 		exit (1);
 	}
 #else
 	if (setgid (j->gid) != 0) {
 		child_err = CHILD_SETGID;
-		write (3, &errno, sizeof (int));
-		write (3, &child_err, sizeof (child_errcode_t));
+		unused = write (3, &errno, sizeof (int));
+		unused = write (3, &child_err, sizeof (child_errcode_t));
 		close (3);
 		exit (1);
 	}
 #if HAVE_SETEGID
 	if (setegid (j->gid) != 0) {
 		child_err = CHILD_SETEGID;
-		write (3, &errno, sizeof (int));
-		write (3, &child_err, sizeof (child_errcode_t));
+		unused = write (3, &errno, sizeof (int));
+		unused = write (3, &child_err, sizeof (child_errcode_t));
 		close (3);
 		exit (1);
 	}
@@ -422,24 +417,24 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 #if HAVE_SETREUID
 	if (setreuid (j->uid, j->uid) != 0) {
 		child_err = CHILD_SETUID;
-		write (3, &errno, sizeof (int));
-		write (3, &child_err, sizeof (child_errcode_t));
+		unused = write (3, &errno, sizeof (int));
+		unused = write (3, &child_err, sizeof (child_errcode_t));
 		close (3);
 		exit (1);
 	}
 #else
 	if (setuid (j->uid) != 0) {
 		child_err = CHILD_SETUID;
-		write (3, &errno, sizeof (int));
-		write (3, &child_err, sizeof (child_errcode_t));
+		unused = write (3, &errno, sizeof (int));
+		unused = write (3, &child_err, sizeof (child_errcode_t));
 		close (3);
 		exit (1);
 	}
 #if HAVE_SETEUID
 	if (seteuid (j->uid) != 0) {
 		child_err = CHILD_SETEUID;
-		write (3, &errno, sizeof (int));
-		write (3, &child_err, sizeof (child_errcode_t));
+		unused = write (3, &errno, sizeof (int));
+		unused = write (3, &child_err, sizeof (child_errcode_t));
 		close (3);
 		exit (1);
 	}
@@ -449,9 +444,9 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 	/* Change dir */
 #if HAVE_CHDIR
 	if (chdir (j->work) != 0) {
-		write (3, &errno, sizeof (int));
-		child_err = CHILD_CHDIR; 
-		write (3, &child_err, sizeof (child_errcode_t));
+		unused = write (3, &errno, sizeof (int));
+		child_err = CHILD_CHDIR;
+		unused = write (3, &child_err, sizeof (child_errcode_t));
 		close (3);
 		exit (1);
 	}
@@ -469,14 +464,16 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 				S_IROTH|S_IWOTH);
 	}
 	if (out < 0) {
-		write (3, &errno, sizeof (int));
-		child_err = CHILD_OPEN; 
-		write (3, &child_err, sizeof (child_errcode_t));
+		unused = write (3, &errno, sizeof (int));
+		child_err = CHILD_OPEN;
+		unused = write (3, &child_err, sizeof (child_errcode_t));
 		close (3);
 		exit (1);
 	}
-	dup2 (out, 1);
-	close (out);
+	if (out != 1) {
+		dup2 (out, 1);
+		close (out);
+	}
 
 	/*
 	 * Open stderr (if not set, open /dev/null.  If the file
@@ -486,9 +483,9 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 	if (j->file_stderr == NULL || (j->file_stderr)[0] == '\0') {
 		err = open ("/dev/null", O_WRONLY);
 		if (err < 0) {
-			write (3, &errno, sizeof (int));
-			child_err = CHILD_OPEN; 
-			write (3, &child_err, sizeof (child_errcode_t));
+			unused = write (3, &errno, sizeof (int));
+			child_err = CHILD_OPEN;
+			unused = write (3, &child_err, sizeof (child_errcode_t));
 			close (1);
 			close (3);
 			exit (1);
@@ -509,16 +506,18 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 					S_IRGRP|S_IWGRP|
 					S_IROTH|S_IWOTH);
 			if (err < 0) {
-				write (3, &errno, sizeof (int));
-				child_err = CHILD_OPEN; 
-				write (3, &child_err,
+				unused = write (3, &errno, sizeof (int));
+				child_err = CHILD_OPEN;
+				unused = write (3, &child_err,
 						sizeof (child_errcode_t));
 				close (1);
 				close (3);
 				exit (1);
 			}
-			dup2 (err, 2);
-			close (err);
+			if (err != 2) {
+				dup2 (err, 2);
+				close (err);
+			}
 		}
 	}
 
@@ -526,9 +525,9 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 	execve (j->command, (j->arguments).arg, (j->environ).env);
 
 	/* Error */
-	write (3, &errno, sizeof (int));
-	child_err = CHILD_EXEC; 
-	write (3, &child_err, sizeof (child_errcode_t));
+	unused = write (3, &errno, sizeof (int));
+	child_err = CHILD_EXEC;
+	unused = write (3, &child_err, sizeof (child_errcode_t));
 	close (1);
 	close (2);
 	close (3);
@@ -627,7 +626,7 @@ exec_command (job_run_t *j, const char *job_id, int *job_pid, char **err_msg)
  * Submit a job
  * If not NULL, job_pid is set with the PID of the runnning job
  * In case of error, if err_msg is not NULL, the error message is returned in
- * *err_msg (it must be freed by the caller). 
+ * *err_msg (it must be freed by the caller).
  * If no error, *err_msg is set to NULL.
  * Warning: even in case of error, *err_msg may be NULL if malloc failed to
  * allocate enough space for this error message.
