@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -18,15 +18,15 @@
 */
 
 /*
- * job_parameters.c -- Excution parameters for a job
+ * job_parameters.c -- Execution parameters for a job
  *
  * The parameters (members of the job_parameters structure) are the
  * following:
  *
  *        command --> The command to be run
  *       username --> User name to use to run the command
- *    file_stdout --> File name of the command stdout redirection 
- *    file_stderr --> File name of the command stderr redirection 
+ *    file_stdout --> File name of the command stdout redirection
+ *    file_stderr --> File name of the command stderr redirection
  *       load_env --> Do the user environement should be load (1) or
  *                    not (0).  If yes (1), the user shell is run with the
  *                    command as a parameter (ie. /bin/sh -c command)
@@ -69,7 +69,7 @@ init_job_parameters (job_parameters_t *j)
 
 
 /*
- * Free the content of a job_paramters structure
+ * Free the content of a job_parameters structure
  */
 void
 destroy_job_parameters (job_parameters_t *j)
@@ -102,10 +102,9 @@ destroy_job_parameters (job_parameters_t *j)
  *   -1 --> Memory allocation error
  */
 static int
-add_attr_job_parameters (char **attr, const char *str)
+add_attr_job_parameters (char **attr, const char *str, size_t len)
 {
 	char *tmp;
-	size_t len;
 
 #if HAVE_ASSERT_H
 	assert (attr != NULL);
@@ -119,8 +118,6 @@ add_attr_job_parameters (char **attr, const char *str)
 		return 0;
 	}
 
-	len = schedwi_strlen (str);
-
 	if (*attr == NULL || len >= schedwi_strlen (*attr)) {
 		tmp = (char *)malloc (len + 1);
 		if (tmp == NULL) {
@@ -131,7 +128,8 @@ add_attr_job_parameters (char **attr, const char *str)
 		}
 		*attr = tmp;
 	}
-	strcpy (*attr, str);
+	strncpy (*attr, str, len);
+	(*attr)[len] = '\0';
 	return 0;
 }
 
@@ -144,13 +142,14 @@ add_attr_job_parameters (char **attr, const char *str)
  *   -1 --> Memory allocation error
  */
 int
-add_command_job_parameters (job_parameters_t *j, const char *command)
+add_command_job_parameters (	job_parameters_t *j, const char *command,
+				size_t len)
 {
 	if (j == NULL) {
 		return 0;
 	}
 
-	return add_attr_job_parameters (&(j->command), command);
+	return add_attr_job_parameters (&(j->command), command, len);
 }
 
 
@@ -162,13 +161,14 @@ add_command_job_parameters (job_parameters_t *j, const char *command)
  *   -1 --> Memory allocation error
  */
 int
-add_username_job_parameters (job_parameters_t *j, const char *username)
+add_username_job_parameters (	job_parameters_t *j, const char *username,
+				size_t len)
 {
 	if (j == NULL) {
 		return 0;
 	}
 
-	return add_attr_job_parameters (&(j->username), username);
+	return add_attr_job_parameters (&(j->username), username, len);
 }
 
 
@@ -180,13 +180,13 @@ add_username_job_parameters (job_parameters_t *j, const char *username)
  *   -1 --> Memory allocation error
  */
 int
-add_stdout_job_parameters (job_parameters_t *j, const char *file)
+add_stdout_job_parameters (job_parameters_t *j, const char *file, size_t len)
 {
 	if (j == NULL) {
 		return 0;
 	}
 
-	return add_attr_job_parameters (&(j->file_stdout), file);
+	return add_attr_job_parameters (&(j->file_stdout), file, len);
 }
 
 
@@ -198,13 +198,13 @@ add_stdout_job_parameters (job_parameters_t *j, const char *file)
  *   -1 --> Memory allocation error
  */
 int
-add_stderr_job_parameters (job_parameters_t *j, const char *file)
+add_stderr_job_parameters (job_parameters_t *j, const char *file, size_t len)
 {
 	if (j == NULL) {
 		return 0;
 	}
 
-	return add_attr_job_parameters (&(j->file_stderr), file);
+	return add_attr_job_parameters (&(j->file_stderr), file, len);
 }
 
 
@@ -228,12 +228,12 @@ add_loadenv_job_parameters (job_parameters_t *j, int flag)
  *   -1 --> Memory allocation error
  */
 int
-add_arg_job_parameters (job_parameters_t *j, const char *arg)
+add_arg_job_parameters (job_parameters_t *j, const char *arg, size_t len)
 {
 	if (j == NULL) {
 		return 0;
 	}
-	return add_arg (&(j->arguments), arg);
+	return add_arg_s (&(j->arguments), arg, len);
 }
 
 
@@ -253,4 +253,22 @@ add_env_job_parameters (job_parameters_t *j, const char *key, const char *val)
 	return add_env (&(j->environ), key, val);
 }
 
+
+/*
+ * Concat the provided environment_t object at the end of the job environment.
+ * Warning: the provided environment_t object will be emptied by this function.
+ *
+ * Return:
+ *    0 --> No error
+ *   -1 --> Memory allocation error
+ */
+int
+concat_env_job_parameters (job_parameters_t *j, environment_t *to_concat)
+{
+	if (j == NULL) {
+		return 0;
+	}
+	return concat_env (&(j->environ), to_concat);
+}
+
 /*-----------------============== End Of File ==============-----------------*/
