@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -18,15 +18,15 @@
 */
 
 /*
- * job_parameters.h -- Excution parameters for a job
+ * job_parameters.h -- Execution parameters for a job
  *
  * The parameters (members of the job_parameters structure) are the
  * following:
  *
  *        command --> The command to be run
  *       username --> User name to use to run the command
- *    file_stdout --> File name of the command stdout redirection 
- *    file_stderr --> File name of the command stderr redirection 
+ *    file_stdout --> File name of the command stdout redirection
+ *    file_stderr --> File name of the command stderr redirection
  *       load_env --> Do the user environement should be load (1) or
  *                    not (0).  If yes (1), the user shell is run with the
  *                    command as a parameter (ie. /bin/sh -c command)
@@ -62,17 +62,24 @@ typedef struct job_parameters job_parameters_t;
 extern void init_job_parameters OF((job_parameters_t *j));
 extern void destroy_job_parameters OF((job_parameters_t *j));
 extern int add_command_job_parameters OF((	job_parameters_t *j,
-						const char *command));
+						const char *command,
+						size_t len));
 extern int add_username_job_parameters OF((	job_parameters_t *j,
-						const char *username));
+						const char *username,
+						size_t len));
 extern int add_stdout_job_parameters OF((	job_parameters_t *j,
-						const char *file));
+						const char *file,
+						size_t len));
 extern int add_stderr_job_parameters OF((	job_parameters_t *j,
-						const char *file));
+						const char *file,
+						size_t len));
 extern void add_loadenv_job_parameters OF((job_parameters_t *j, int flag));
-extern int add_arg_job_parameters OF((job_parameters_t *j, const char *arg));
+extern int add_arg_job_parameters OF((	job_parameters_t *j, const char *arg,
+					size_t len));
 extern int add_env_job_parameters OF((	job_parameters_t *j,
 					const char *key, const char *val));
+extern int concat_env_job_parameters OF((job_parameters_t *j,
+					environment_t *to_concat));
 
 #ifdef __cplusplus
 }
