@@ -1,6 +1,6 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
-     
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -24,8 +24,8 @@
  * following:
  *
  *        command --> The command to be run
- *    file_stdout --> File name of the command stdout redirection 
- *    file_stderr --> File name of the command stderr redirection 
+ *    file_stdout --> File name of the command stdout redirection
+ *    file_stderr --> File name of the command stderr redirection
  *           work --> Working directory (a chdir(2) will be done in this
  *                    directory before running (exec) the user command)
  *          shell --> User default shell
@@ -40,10 +40,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -435,7 +431,7 @@ parse_login_defaults (mode_t *mask, rlim_t *ulim, char **path)
  * Given a /etc/password GECOS field, and parsing the /etc/login.defs (or
  * /etc/default/login), fill the following members of the job_run structure:
  *
- *          mask --> umask 
+ *          mask --> umask
  *           lim --> ulimit
  *          nice --> scheduling priority (nice) retrieved only from the
  *                   gecos field
@@ -464,7 +460,7 @@ parse_gecos (const char *gecos, job_run_t *j)
 #endif
 
 	/* Get the values from /etc/login.defs or /etc/default/login */
-	ret = parse_login_defaults (&umsk, &ulim, &path); 
+	ret = parse_login_defaults (&umsk, &ulim, &path);
 	if (ret == -1) {
 		return -1;
 	}
@@ -544,7 +540,7 @@ parse_gecos (const char *gecos, job_run_t *j)
  *           gid --> User group ID
  *          mask --> umask for this user
  *           lim --> ulimit for this user
- *          nice --> scheduling priority (nice) for this user 
+ *          nice --> scheduling priority (nice) for this user
  *       environ --> Initial environment for this user (SHELL, PATH, LOGNAME,
  *                   USER, HOME)
  *
@@ -732,7 +728,7 @@ add_outfile_job_run (	job_run_t *j, const char *file_stdout,
 		}
 		max += 100;
 #if HAVE_STRFTIME && HAVE_LOCALTIME && HAVE_TIME
-		time_len = strftime (out1, max, out, curr_tm); 
+		time_len = strftime (out1, max, out, curr_tm);
 #else
 		strncpy (out1, out, max);
 		time_len = max - 1;
@@ -779,7 +775,7 @@ add_outfile_job_run (	job_run_t *j, const char *file_stdout,
 		max += 100;
 
 #if HAVE_STRFTIME && HAVE_LOCALTIME && HAVE_TIME
-		time_len = strftime (err1, max, out, curr_tm); 
+		time_len = strftime (err1, max, out, curr_tm);
 #else
 		strncpy (err1, out, max);
 		time_len = max - 1;
