@@ -1,6 +1,6 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
-     
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -24,8 +24,8 @@
  * following:
  *
  *        command --> The command to be run
- *    file_stdout --> File name of the command stdout redirection 
- *    file_stderr --> File name of the command stderr redirection 
+ *    file_stdout --> File name of the command stdout redirection
+ *    file_stderr --> File name of the command stderr redirection
  *           work --> Working directory (a chdir(2) will be done in this
  *                    directory before running (exec) the user command)
  *          shell --> User default shell
@@ -43,10 +43,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
