@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -44,6 +44,24 @@ extern int errno;
 #include <net_module.h>
 #include <lwc_log.h>
 #include <net_parse.h>
+#include <JSON_parser.h>
+
+
+/*
+ * JSON callback function
+ */
+static int
+json_char_callback (void* ctx, int type, const JSON_value* value)
+{
+	char *s = (char *)ctx;
+
+	if (type == JSON_T_STRING) {
+		strncpy (s, value->vu.str.value, SCHEDWI_NET_FUNCTION_LENGHT);
+		s[SCHEDWI_NET_FUNCTION_LENGHT] = '\0';
+	}
+	return 1;
+}
+
 
 /*
  * Parse and run the network request
@@ -60,6 +78,10 @@ net_parse (net_id *sock, char **buff, size_t *len)
 	ssize_t nb_read;
 	char module[SCHEDWI_NET_FUNCTION_LENGHT + 1];
 	int (*function)(net_id *, char *, size_t);
+	int i;
+	JSON_config config;
+	struct JSON_parser_struct *jc;
+
 
 #if HAVE_ASSERT_H
 	assert (sock != NULL && buff != NULL && len != NULL);
@@ -70,14 +92,35 @@ net_parse (net_id *sock, char **buff, size_t *len)
 		return -1;
 	}
 
-	if (nb_read < SCHEDWI_NET_FUNCTION_LENGHT) {
+	/* JSON parser initialization */
+	init_JSON_config (&config);
+
+	module[0]                     = '\0';
+	config.depth                  = 20;
+	config.callback               = &json_char_callback;
+	config.allow_comments         = 0;
+	config.handle_floats_manually = 0;
+	config.callback_ctx           = module;
+
+	jc = new_JSON_parser (&config);
+
+	/* Look for the first JSON string which is the module name */
+	for (i = 0; i < nb_read && module[0]  == '\0'; i++) {
+		if (! JSON_parser_char (jc, (*buff)[i])) {
+			lwc_writeLog (LOG_ERR,
+			_("Wrong network request: JSON syntax error"));
+			delete_JSON_parser (jc);
+			return -1;
+		}
+	}
+	delete_JSON_parser (jc);
+
+	if (module[0] == '\0') {
 		lwc_writeLog (LOG_ERR,
-			_("Wrong network request: not enough characters"));
+			_("Wrong network request: module name not specified"));
 		return -1;
 	}
 
-	strncpy (module, *buff, SCHEDWI_NET_FUNCTION_LENGHT);
-	module[SCHEDWI_NET_FUNCTION_LENGHT] = '\0';
 	function = net_module (module);
 	if (function == NULL) {
 		lwc_writeLog (LOG_ERR,
@@ -86,10 +129,7 @@ net_parse (net_id *sock, char **buff, size_t *len)
 		return -1;
 	}
 
-	if (function (	sock,
-			*buff + SCHEDWI_NET_FUNCTION_LENGHT,
-			nb_read - SCHEDWI_NET_FUNCTION_LENGHT) != 0)
-	{
+	if (function (sock, *buff, nb_read) != 0) {
 		return -1;
 	}
 	return 0;
