@@ -1,6 +1,6 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
-     
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -29,7 +29,6 @@
 #include <assert.h>
 #endif
 
-#include <conf.h>
 #include <net_utils_init.h>
 
 static char enable_ssl;
@@ -45,21 +44,14 @@ static char enable_ssl;
  */
 int
 net_init (	const char *client_crt, const char *client_key,
-		const char *server_crt, char quick_random)
+		const char *server_crt, char quick_random, char ssl)
 {
+	enable_ssl = ssl;
 #if HAVE_LIBGNUTLS
-	int ret;
-
-	ret = conf_get_param_bool ("SSLEngine", &enable_ssl);
-#if HAVE_ASSERT_H
-	assert (ret == 0);
-#endif
 	if (enable_ssl != 0) {
 		return net_init_ssl (	client_crt, client_key,
 					server_crt, quick_random);
 	}
-#else
-	enable_ssl = 0;
 #endif
 	return 0;
 }
