@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -28,8 +28,8 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
 #endif
 
 /* Allow the use in C++ code.  */
@@ -37,13 +37,16 @@
 extern "C" {
 #endif
 
-extern int net_server_sock OF((const char *port_number, const char *iface));
-extern int net_accept_sock OF((int sock, const char *schedwi_server_name));
+extern char *port2number OF((const char *port));
+extern int net_server_sock OF((const char *port_number, const char *iface,
+				const char *address_family,
+				fd_set *sock_set, int *sock_max));
+extern int net_accept_sock OF((fd_set *sock_set, int sock_max,
+				const char *allow_from));
 extern int net_client_sock OF((const char *port_number, const char *hostname));
 extern ssize_t net_read_sock OF((int sock, char **buff, size_t *len));
 extern int net_read_result_sock OF((int sock, char **result_msg));
 extern int net_write_sock OF((int sock, const char *buff, size_t len));
-extern int net_write_eof_sock OF((int sock));
 extern int net_close_sock OF((int sock));
 
 #ifdef __cplusplus
