@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2011 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -32,33 +32,10 @@
 #include <unistd.h>
 #endif
 
-#if HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
-
-#if HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-
-#if HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
-
-#if HAVE_NETDB_H
-#include <netdb.h>
-#endif
-
 #if HAVE_TIME_H
 #include <time.h>
 #endif
 
-#if HAVE_ERRNO_H
-#include <errno.h>
-#endif
-#ifndef errno
-extern int errno;
-#endif
-
 #if HAVE_ASSERT_H
 #include <assert.h>
 #endif
@@ -71,9 +48,11 @@ extern int errno;
 #include <lwc_log.h>
 #include <net_utils_sock.h>
 #include <net_utils_ssl.h>
+#include <JSON_parser.h>
+#include <parse_json_result.h>
+
 
 #define BUFFER_LEN 1024
-#define ENF_OF_REQUEST "\nbYe"
 
 
 /* Credentials (client certificate and key and server certificate) */
@@ -88,6 +67,9 @@ static gnutls_rsa_params_t rsa_params;
 
 static char params_set = 0;
 
+/* Wether GnuTLS has been initialized (1) or not (0) */
+static char gnutls_initialized = 0;
+
 
 /*
  * Initialize the Diffie Hellman and RSA algorithms (only required for server)
@@ -101,8 +83,6 @@ net_init_server_ssl ()
 {
 	int ret;
 
-	lwc_writeLog (LOG_INFO,
-		_("Initializing the SSL layer (may take several minutes)..."));
 
 	/* Build the Diffie Hellman parameters */
 	ret = gnutls_dh_params_init (&dh_params);
@@ -149,7 +129,38 @@ _("SSL: Failed to generate the parameters for the RSA algorithms: %s"),
 	}
 
 	params_set = 1;
-	lwc_writeLog (LOG_INFO, _("Done"));
+	return 0;
+}
+
+
+/*
+ * Initialize the GnuTLS layer
+ *
+ * Return:
+ *   0 --> OK
+ *  -1 --> Error (a message has been logged)
+ */
+int
+net_init_gnutls (char quick_random)
+{
+	int ret;
+
+
+	/* GNUTLS init */
+	ret = gnutls_global_init ();
+	if (ret != 0) {
+		lwc_writeLog (LOG_CRIT,
+			_("SSL: Cannot initialize the SSL library: %s"),
+			gnutls_strerror (ret));
+		return -1;
+	}
+
+	/* Fast random mode (less secure) */
+	if (quick_random != 0) {
+		gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
+	}
+
+	gnutls_initialized = 1;
 	return 0;
 }
 
@@ -172,11 +183,6 @@ net_init_ssl (	const char *clnt_crt_file, const char *clnt_key_file,
 		&& server_crt != NULL);
 #endif
 
-	/* Fast random mode (less secure) */
-	if (quick_random != 0) {
-		gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
-	}
-
 	if (cerdentials_set != 0) {
 		gnutls_certificate_free_credentials (x509_cred);
 		cerdentials_set = 0;
@@ -189,11 +195,7 @@ net_init_ssl (	const char *clnt_crt_file, const char *clnt_key_file,
 	}
 
 	/* GNUTLS init */
-	ret = gnutls_global_init ();
-	if (ret != 0) {
-		lwc_writeLog (LOG_CRIT,
-			_("SSL: Cannot initialize the SSL library: %s"),
-			gnutls_strerror (ret));
+	if (gnutls_initialized == 0 && net_init_gnutls (quick_random) != 0) {
 		return -1;
 	}
 
@@ -222,7 +224,7 @@ net_init_ssl (	const char *clnt_crt_file, const char *clnt_key_file,
 						GNUTLS_X509_FMT_DER) != 0)
 	{
 		lwc_writeLog (LOG_ERR,
-_("SSL: Cannot load the client certificat `%s' with the key `%s': %s"),
+_("SSL: Cannot load the agent certificat `%s' with the key `%s': %s"),
 				clnt_crt_file, clnt_key_file,
 				gnutls_strerror (ret));
 		gnutls_certificate_free_credentials (x509_cred);
@@ -276,89 +278,6 @@ net_destroy_ssl ()
 }
 
 
-/*
- * Compare two hostnames
- *
- * Return:
- *   0 --> The two hosts are the same
- *   1 --> The two hosts are not the same
- *  -1 --> Error (a message is displayed using lwc_writeLog())
- */
-static
-int hostname_cmp (const char *h1, const char *h2)
-{
-	int in_num_dot, i, try_no;
-	struct hostent *he;
-	struct in_addr in;
-#if HAVE_NANOSLEEP
-	struct timespec req;
-
-	req.tv_sec = SCHEDWI_HOSTLOOKUP_SLEEP;
-	req.tv_nsec = 0;
-#endif
-
-	if (h1 == NULL || h2 == NULL) {
-		return 1;
-	}
-
-	if (schedwi_strcasecmp (h1, h2) == 0) {
-		return 0;
-	}
-
-	/* If the hostname is in number-dot notation, convert it */
-	in_num_dot = inet_aton (h1, &in);
-
-	try_no = 0;
-	do {
-		if (in_num_dot == 0) {
-			he = gethostbyname (h1);
-		}
-		else {
-			he = gethostbyaddr (&in, sizeof (in), AF_INET);
-		}
-		if (he == NULL) {
-			if (h_errno == TRY_AGAIN) {
-				try_no++;
-				if (try_no < SCHEDWI_HOSTLOOKUP_RETRIES) {
-#if HAVE_NANOSLEEP
-					nanosleep (&req, NULL);
-#elif HAVE_SLEEP
-					sleep (SCHEDWI_HOSTLOOKUP_SLEEP);
-#elif HAVE_USLEEP
-					usleep (  SCHEDWI_HOSTLOOKUP_SLEEP
-						* 1000000);
-#endif
-				}
-			}
-			else {
-			if (h_errno == NO_RECOVERY) {
-				lwc_writeLog (	LOG_ERR,
-						_("Name server error"));
-				return -1;
-			}
-			else {
-				return 1;
-			}}
-		}
-	} while (he == NULL && try_no < SCHEDWI_HOSTLOOKUP_RETRIES);
-	if (he == NULL) {
-		lwc_writeLog (LOG_WARNING, _("Name server unavailable"));
-		return -1;
-	}
-
-	if (schedwi_strcasecmp (he->h_name, h2) == 0) {
-		return 0;
-	}
-
-	for (i = 0; (he->h_aliases)[i] != NULL; i++) {
-		if (schedwi_strcasecmp ((he->h_aliases)[i], h2) == 0) {
-			return 0;
-		}
-	}
-	return 1;
-}
-
-
 /*
  * Verify the remote certificate
  *
@@ -367,16 +286,14 @@ int hostname_cmp (const char *h1, const char *h2)
  *  -1 --> Error (an error message is displayed using lwc_writeLog())
  */
 static int
-net_utils_ssl_check_certificate (gnutls_session_t *session, const char *hostname)
+net_utils_ssl_check_certificate (	gnutls_session_t *session,
+					const char *hostname)
 {
-	int ret, i;
+	int ret;
 	unsigned int status, cert_list_size;
 	gnutls_x509_crt_t cert;
 	const gnutls_datum_t *cert_list;
 	time_t now;
-	char dnsname[512];
-	size_t dnsnamesize;
-	char found_dnsname;
 
 #if HAVE_ASSERT_H
 	assert (hostname != NULL && session != NULL);
@@ -459,7 +376,19 @@ net_utils_ssl_check_certificate (gnutls_session_t *session, const char *hostname
 		return -1;
 	}
 
-	/* Log a message if the certificate expires in less than a month */
+	/* Check the hostname */
+	if (gnutls_x509_crt_check_hostname (cert, hostname) == 0) {
+		lwc_writeLog (LOG_ERR,
+			_("SSL: The certificate hostname does not match %s."),
+			hostname);
+		gnutls_x509_crt_deinit (cert);
+		return -1;
+	}
+
+	/*
+	 * Log an information message if the certificate expires in less
+	 * than a month
+	 */
 	if (gnutls_x509_crt_get_expiration_time (cert)
 			< now + 30 * 24 * 60 * 60)
 	{
@@ -468,51 +397,8 @@ _("SSL: The certificate for %s is going to expire in less than a month."),
 				hostname);
 	}
 
-	/*
-	 * Check the hostname.  One could use the provided
-	 * gnutls_x509_crt_check_hostname() function.  However, we will check
-	 * all the known names of the host (from the DNS, /etc/hosts, ...)
-	 * The following lines are then inspired from this function
-	 * in GNUTLS (see rfc2818_hostname.c)
-	 */
-	found_dnsname = 0;
-	for (i = ret = 0; ret >= 0; i++) {
-		dnsnamesize = sizeof (dnsname);
-		ret = gnutls_x509_crt_get_subject_alt_name (cert, i,
-						dnsname, &dnsnamesize, NULL);
-		if (ret == GNUTLS_SAN_DNSNAME) {
-			found_dnsname = 1;
-			if (hostname_cmp (hostname, dnsname) == 0) {
-				gnutls_x509_crt_deinit (cert);
-				return 0;
-			}
-		}
-	}
-
-	if (found_dnsname == 0) {
-		dnsnamesize = sizeof (dnsname);
-		ret = gnutls_x509_crt_get_dn_by_oid (cert,
-						GNUTLS_OID_X520_COMMON_NAME,
-						0, 0,
-						dnsname, &dnsnamesize);
-		if (ret < 0) {
-			lwc_writeLog (LOG_ERR,
-		_("SSL: Cannot retrieve the certificate hostname for %s: %s"),
-				hostname, gnutls_strerror (ret));
-			gnutls_x509_crt_deinit (cert);
-			return -1;
-		}
-		if (hostname_cmp (hostname, dnsname) == 0) {
-			gnutls_x509_crt_deinit (cert);
-			return 0;
-		}
-	}
-
 	gnutls_x509_crt_deinit (cert);
-	lwc_writeLog (	LOG_ERR,
-			_("SSL: The certificate hostname does not match %s."),
-			hostname);
-	return -1;
+	return 0;
 }
 
 
@@ -530,7 +416,6 @@ int
 net_accept_ssl (int fd, gnutls_session_t *session, const char *server_hostname)
 {
 	int ret;
-	const int cert_type_priority[2] = { GNUTLS_CRT_X509, 0 };
 
 #if HAVE_ASSERT_H
 	assert (session != NULL && server_hostname != NULL);
@@ -547,7 +432,7 @@ net_accept_ssl (int fd, gnutls_session_t *session, const char *server_hostname)
 
 	/* Use default priority */
 	gnutls_set_default_priority (*session);
-	gnutls_certificate_type_set_priority (*session, cert_type_priority);
+	gnutls_priority_set_direct (*session, "NORMAL", NULL);
 
 	/* Put the x509 credentials to the current session */
 	ret = gnutls_credentials_set (	*session, GNUTLS_CRD_CERTIFICATE,
@@ -603,7 +488,6 @@ int
 net_client_ssl (int fd, gnutls_session_t *session, const char *server_hostname)
 {
 	int ret;
-	const int cert_type_priority[2] = { GNUTLS_CRT_X509, 0 };
 
 #if HAVE_ASSERT_H
 	assert (server_hostname != NULL && session != NULL);
@@ -620,7 +504,7 @@ net_client_ssl (int fd, gnutls_session_t *session, const char *server_hostname)
 
 	/* Use default priority */
 	gnutls_set_default_priority (*session);
-	gnutls_certificate_type_set_priority (*session, cert_type_priority);
+	gnutls_priority_set_direct (*session, "NORMAL", NULL);
 
 	/* Put the x509 credentials to the current session */
 	ret = gnutls_credentials_set (	*session, GNUTLS_CRD_CERTIFICATE,
@@ -659,6 +543,33 @@ net_client_ssl (int fd, gnutls_session_t *session, const char *server_hostname)
 }
 
 
+/*
+ * JSON callback function
+ */
+static int
+json_char_callback (void* ctx, int type, const JSON_value* value)
+{
+	int *nb_open_brackets = (int *)ctx;
+
+	switch (type) {
+		case JSON_T_ARRAY_BEGIN:
+		case JSON_T_OBJECT_BEGIN:
+			if (*nb_open_brackets < 0) {
+				*nb_open_brackets = 1;
+			}
+			else {
+				(*nb_open_brackets)++;
+			}
+			break;
+		case JSON_T_ARRAY_END:
+		case JSON_T_OBJECT_END:
+			(*nb_open_brackets)--;
+			break;
+	}
+	return 1;
+}
+
+
 /*
  * Read data from the provided SSL session handler.
  * The data are store in *buff (of size *len). If buffer is NULL or not big
@@ -676,6 +587,10 @@ net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 	ssize_t nb_read, total;
 	size_t to_read;
 	char *b, *tmp;
+	int nb_open_brackets, i;
+	JSON_config config;
+	struct JSON_parser_struct *jc;
+
 
 #if HAVE_ASSERT_H
 	assert (buff != NULL && len != NULL);
@@ -691,12 +606,25 @@ net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 		*len = BUFFER_LEN;
 	}
 
+	/* JSON parser initialization */
+	init_JSON_config (&config);
+
+	nb_open_brackets              = -1;
+	config.depth                  = 20;
+	config.callback               = &json_char_callback;
+	config.allow_comments         = 0;
+	config.handle_floats_manually = 0;
+	config.callback_ctx           = &nb_open_brackets;
+
+	jc = new_JSON_parser (&config);
+
 	total = 0;
 	to_read = *len;
 	b = *buff;
 	do {
 		nb_read = gnutls_record_recv (session, b, to_read);
 		if (nb_read == 0) {
+			delete_JSON_parser (jc);
 			return total;
 		}
 
@@ -707,6 +635,7 @@ net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 				lwc_writeLog (LOG_ERR,
 					_("Error while reading data: %s"),
 					gnutls_strerror (nb_read));
+				delete_JSON_parser (jc);
 				return nb_read;
 			}
 			continue;
@@ -714,13 +643,25 @@ net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 
 		total += nb_read;
 
-		/* Find the EOF tag */
-		tmp = schedwi_strnstr (*buff, ENF_OF_REQUEST, total);
-		if (tmp != NULL) {
-			return (ssize_t)(tmp - *buff);
+		/*
+		 * Count the numbers of open/close brackets.  If this number
+		 * drops to zero it means that the full JSON request has
+		 * been receved.
+		 */
+		for (i = 0; i < nb_read; i++) {
+			if (! JSON_parser_char (jc, b[i])) {
+				lwc_writeLog (	LOG_ERR,
+					_("Syntax error in received data"));
+				delete_JSON_parser (jc);
+				return -1;
+			}
+			if (nb_open_brackets == 0) {
+				delete_JSON_parser (jc);
+				return total;
+			}
 		}
 
-		if (nb_read < to_read) {
+		if ((size_t)nb_read < to_read) {
 			to_read -= nb_read;
 			b += nb_read;
 		}
@@ -729,6 +670,7 @@ net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 				/* Too many data */
 				lwc_writeLog (LOG_NOTICE,
 		_("Failed to read the network request: buffer overflow"));
+				delete_JSON_parser (jc);
 				return -2;
 			}
 
@@ -736,6 +678,7 @@ net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 			if (tmp == NULL) {
 				lwc_writeLog (	LOG_CRIT,
 						_("Memory allocation error"));
+				delete_JSON_parser (jc);
 				return -1;
 			}
 			*buff = tmp;
@@ -745,6 +688,8 @@ net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 		}
 	} while (1);
 
+	delete_JSON_parser (jc);
+
 	return total;
 }
 
@@ -752,10 +697,12 @@ net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 /*
  * Read the reply
  *
- * The format of the reply must be as follow:
- *    The first character: '0' means success, an other character means failure
- *    The next characters: An optional explanation message (which is returned
- *                         in result_msg)
+ * The format of the JSON reply looks like:
+ *     {
+ *         "success" : false,
+ *          "data" : "Database unavailable"
+ *     }
+ *
  * No reply (nothing to read) means failure.
  *
  * Return:
@@ -770,10 +717,10 @@ net_read_ssl (gnutls_session_t session, char **buff, size_t *len)
 int
 net_read_result_ssl (gnutls_session_t session, char **result_msg)
 {
-	char *buff, ret_code;
+	char *buff;
 	size_t len;
 	ssize_t nb_read;
-	int i;
+	int ret;
 
 	if (result_msg != NULL) {
 		*result_msg = NULL;
@@ -791,32 +738,15 @@ net_read_result_ssl (gnutls_session_t session, char **result_msg)
 	}
 
 	if (nb_read > 0) {
-		/* Return code sent by the remote process */
-		ret_code = buff[0];
-
-		if (result_msg != NULL) {
-			/* Shift the message 1 character to the left */
-			for (i = 1; i < nb_read; i++) {
-				buff[i - 1] = buff[i];
-			}
-			buff [nb_read - 1] = '\0';
-			*result_msg = buff;
-		}
-		else {
-			free (buff);
-		}
-
-		/* Return code '0' means success */
-		if (ret_code == '0') {
-			return 0;
-		}
+		ret = parse_json_result (buff, nb_read, result_msg, NULL);
 	}
 	else {
-		if (buff != NULL) {
-			free (buff);
-		}
+		ret = 1;
 	}
-	return 1;
+	if (buff != NULL) {
+		free (buff);
+	}
+	return ret;
 }
 
 
@@ -838,7 +768,7 @@ net_write_ssl (gnutls_session_t session, const char *buff, size_t len)
 
 	do {
 		nb_write = gnutls_record_send (session, buff, len);
-		if (nb_write == len) {
+		if ((size_t)nb_write == len) {
 			return 0;
 		}
 
@@ -864,22 +794,7 @@ net_write_ssl (gnutls_session_t session, const char *buff, size_t len)
 
 
 /*
- * Write the End of Request tag
- *
- * Return:
- *   0 --> No error
- *  -1 --> Write error (a message is displayed using lwc_writeLog())
- */
-int
-net_write_eof_ssl (gnutls_session_t session)
-{
-	return net_write_ssl (session, ENF_OF_REQUEST,
-				(size_t)schedwi_strlen (ENF_OF_REQUEST));  
-}
-
-
-/*
- * Close the SSL handler 
+ * Close the SSL handler
  */
 int
 net_close_ssl (gnutls_session_t session, int fd)
