@@ -1,6 +1,6 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
-     
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -26,10 +26,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #include <gnutls/gnutls.h>
 
 /* Allow the use in C++ code.  */
@@ -37,6 +33,7 @@
 extern "C" {
 #endif
 
+extern int net_init_gnutls OF((char quick_random));
 extern int net_init_ssl OF((	const char *clnt_crt_file,
 				const char *clnt_key_file,
 				const char *server_crt,
@@ -52,7 +49,6 @@ extern ssize_t net_read_ssl OF((gnutls_session_t session,
 extern int net_read_result_ssl OF((gnutls_session_t session, char **result_msg));
 extern int net_write_ssl OF((	gnutls_session_t session,
 				const char *buff, size_t len));
-extern int net_write_eof_ssl OF((gnutls_session_t session));
 extern int net_close_ssl OF((gnutls_session_t session, int fd));
 
 #ifdef __cplusplus
