@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2011 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -21,10 +21,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -75,9 +71,11 @@ extern int errno;
 #include <lib_functions.h>
 #include <net_utils_init.h>
 #include <net_utils_glob.h>
+#include <net_utils_ssl.h>
 #include <net_parse.h>
 #include <net_module.h>
 #include <net_module_clnt.h>
+#include <cert_client.h>
 
 static const char *pid_file = NULL;
 static const char *configuration_file = NULL;
@@ -170,6 +168,12 @@ sighup_action (int sig)
 	close_pid_file (pid_file);
 	conf_destroy_clnt ();
 
+	/*
+	 * Send the TERM signal to the `send_proccess' process.
+	 * A KILL signal will be also send later (see a little bit further)
+	 */
+	send_process_term ();
+
 	/*
 	 * Re-read the configuration file.
 	 * WARNING an error in this file (missing file or syntax error) will
@@ -244,6 +248,19 @@ sighup_action (int sig)
 				_("An other daemon is already running"));
 			break;
 	}
+
+	/*
+	 * Re-start the `send_process' process
+	 */
+	send_process_kill ();
+	if (send_process_start () != 0) {
+		lwc_closeLog ();
+		close_pid_file (pid_file);
+		conf_destroy_clnt ();
+		destroy_modules ();
+		net_free ();
+		exit (1);
+	}
 }
 
 
@@ -297,11 +314,11 @@ main (int argc, char **argv)
 	size_t buff_len;
 	int ret;
 	struct sigaction sa;
-	const char *log_file, *date_format;
-	const char *iface, *agent_port, *server;
+	const char *log_file, *date_format, *registrar_port;
+	const char *iface, *agent_port, *address_family, *server, *allow_from;
 	long int facility;
 	const char *client_crt, *client_key, *server_crt;
-	char quick_random, test, foreground;
+	char quick_random, test, foreground, enable_ssl;
 	const char *prog_name;
 
 #if HAVE_GETOPT_LONG
@@ -406,21 +423,18 @@ main (int argc, char **argv)
 	 * Initialize the network (before detaching the program from its
 	 * terminal as a password may be asked for the SSL private key)
 	 */
-	ret = conf_get_param_string ("SSLClientCertificateFile", &client_crt);
-	ret += conf_get_param_string (	"SSLClientCertificateKeyFile",
-					&client_key);
-	ret += conf_get_param_string ("SSLServerCertificateFile", &server_crt);
+	ret = conf_get_param_string ("SSLCertificateFile", &client_crt);
+	ret += conf_get_param_string ("SSLCertificateKeyFile", &client_key);
+	ret += conf_get_param_string ("SSLCACertificateFile", &server_crt);
 	ret += conf_get_param_bool ("SSLQuickRandom", &quick_random);
+	ret += conf_get_param_bool ("SSLEngine", &enable_ssl);
+	ret += conf_get_param_string ("SERVER", &server);
+	ret += conf_get_param_string ("AGENT_PORT", &agent_port);
+	ret += conf_get_param_string ("REGISTRAR_PORT", &registrar_port);
 #if HAVE_ASSERT_H
 	assert (ret == 0);
 #endif
 
-	if (net_init (client_crt, client_key, server_crt, quick_random) != 0) {
-		conf_destroy_clnt ();
-		destroy_modules ();
-		return 1;
-	}
-
 
 	/* It was only a test of the configuration file */
 	if (test == 1) {
@@ -438,7 +452,6 @@ main (int argc, char **argv)
 			perror ("fork");
 			conf_destroy_clnt ();
 			destroy_modules ();
-			net_free ();
 			return 1;
 		}
 	}
@@ -493,7 +506,49 @@ main (int argc, char **argv)
 		lwc_closeLog ();
 		conf_destroy_clnt ();
 		destroy_modules ();
-		net_free ();
+		return 1;
+	}
+
+
+#if HAVE_LIBGNUTLS
+	/*
+	 * Initialize the GnuTLS layer
+	 */
+	if (enable_ssl != 0 && net_init_gnutls (quick_random) != 0) {
+		conf_destroy_clnt ();
+		destroy_modules ();
+		return 1;
+	}
+#endif
+
+
+	/*
+	 * Register this agent with the server if not already done
+	 */
+	if (client_cert_check_and_create (	enable_ssl, server_crt,
+						client_crt, client_key,
+						quick_random,
+						server, registrar_port,
+						agent_port) != 0)
+	{
+		lwc_closeLog ();
+		close_pid_file (pid_file);
+		conf_destroy_clnt ();
+		destroy_modules ();
+		return 1;
+	}
+
+
+	/*
+	 * Initialize the network
+	 */
+	if (net_init (	client_crt, client_key, server_crt,
+			quick_random, enable_ssl) != 0)
+	{
+		lwc_closeLog ();
+		close_pid_file (pid_file);
+		conf_destroy_clnt ();
+		destroy_modules ();
 		return 1;
 	}
 
@@ -501,6 +556,12 @@ main (int argc, char **argv)
 	/*
 	 * SSL setup
 	 */
+#if HAVE_LIBGNUTLS
+	if (enable_ssl != 0 && quick_random == 0) {
+		lwc_writeLog (LOG_INFO,
+		_("Initializing the SSL layer (may take several minutes)..."));
+	}
+#endif
 	if (net_init_server () != 0) {
 		lwc_closeLog ();
 		close_pid_file (pid_file);
@@ -509,6 +570,11 @@ main (int argc, char **argv)
 		net_free ();
 		return 1;
 	}
+#if HAVE_LIBGNUTLS
+	if (enable_ssl != 0 && quick_random == 0) {
+		lwc_writeLog (LOG_INFO, _("Done"));
+	}
+#endif
 
 
 	/*
@@ -601,12 +667,12 @@ main (int argc, char **argv)
 	 * Network setup
 	 */
 	ret = conf_get_param_string ("IFACE_LISTEN", &iface);
-	ret += conf_get_param_string ("AGENT_PORT", &agent_port);
+	ret += conf_get_param_string ("ADDRESS_FAMILY", &address_family);
 #if HAVE_ASSERT_H
 	assert (ret == 0);
 #endif
 
-	if (net_server (&sock, agent_port, iface) != 0) {
+	if (net_server (&sock, agent_port, iface, address_family) != 0) {
 		send_process_stop ();
 		lwc_closeLog ();
 		close_pid_file (pid_file);
@@ -616,6 +682,7 @@ main (int argc, char **argv)
 		return 1;
 	}
 
+	lwc_writeLog (LOG_INFO, _("Ready"));
 
 	/*
 	 * Main loop
@@ -624,12 +691,16 @@ main (int argc, char **argv)
 	buff_len = 0;
 	do {
 		ret = conf_get_param_string ("SERVER", &server);
+		ret += conf_get_param_string ("ALLOW_FROM", &allow_from);
 #if HAVE_ASSERT_H
 	assert (ret == 0);
 #endif
+		if (allow_from[0] == '\0') {
+			allow_from = server;
+		}
 
 		/* Wait for connections */
-		ret = net_accept (&sock, &s, server);
+		ret = net_accept (&sock, &s, server, allow_from);
 		if (ret < 0) {
 			send_process_stop ();
 			lwc_closeLog ();
